package org.openntf.wf.xsp.providers;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.openntf.wf.kernel.IWorkflowVariableProvider;
import org.openntf.wf.kernel.WorkflowRuntimeException;
import org.openntf.wf.kernel.WorkflowVariable;

import lotus.domino.Database;
import lotus.domino.Document;
import lotus.domino.DocumentCollection;
import lotus.domino.NotesException;
import lotus.domino.View;

public abstract class AbstractNotesVariableStorageProvider implements IWorkflowVariableProvider {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final String VARIABLE_VIEW = "lupWFVariableByWFHeaderId";
	private List<WorkflowVariable> globalVariables;

	@Override
	public List<WorkflowVariable> getAllByWorkflowId(String wfID) {
		List<WorkflowVariable> variables = new ArrayList<WorkflowVariable>();
		Database varDatabase = null;
		View viwVariables = null;
		try {
			varDatabase = getVariableDatabase();
			viwVariables = varDatabase.getView(VARIABLE_VIEW);
			DocumentCollection docCol = viwVariables.getAllDocumentsByKey(wfID, true);
			Document docNext = docCol.getFirstDocument();
			while (docNext != null) {
				Document docProcess = docNext;
				docNext = docCol.getNextDocument();
				WorkflowVariable wfVariable = buildVariable(docProcess);
				variables.add(wfVariable);
			}
		} catch (NotesException ex) {
			throw new WorkflowRuntimeException("Error during read of WorkflowIds", ex);
		}
		return variables;
	}

	@Override
	public List<WorkflowVariable> getGlobalVariables() {
		if (globalVariables == null) {
			buildGlobalVariables();
		}
		return Collections.unmodifiableList(globalVariables);
	}

	private synchronized void buildGlobalVariables() {
		if (globalVariables == null) {
			globalVariables = new ArrayList<WorkflowVariable>();
			globalVariables.add(buildGVEntry("stepname", "wf.stepName"));
			globalVariables.add(buildGVEntry("workflowname", "wf.workflowName"));
			globalVariables.add(buildGVEntry("applicationname", "wf.applicationName"));
		}
		
	}

	private WorkflowVariable buildGVEntry(String name, String espression) {
		WorkflowVariable wfVar = new WorkflowVariable();
		wfVar.setExpression(espression);
		wfVar.setVariablName(name);
		return wfVar;
	}

	protected abstract Database getVariableDatabase() throws NotesException;

	private WorkflowVariable buildVariable(Document doc) throws NotesException {
		WorkflowVariable var = new WorkflowVariable();
		var.setVariablName(doc.getItemValueString("variableName"));
		var.setExpression(doc.getItemValueString("variableValue"));
		return var;
	}
}
